@@ -11,7 +11,7 @@ use 5.014002;
 use strict;
 use warnings;
 
-use version; our $VERSION = version->declare( "v3.0.1" );
+use version; our $VERSION = version->declare( "v3.0.4" );
 
 use Locale::TextDomain 'Zonemaster-CLI';
 use Moose;
@@ -23,6 +23,7 @@ use Zonemaster::Engine::Translator;
 use Zonemaster::Engine::Util qw[pod_extract_for];
 use Zonemaster::Engine::Exception;
 use Zonemaster::Engine::Zone;
+use Zonemaster::Engine::Net::IP;
 use Scalar::Util qw[blessed];
 use Encode;
 use Zonemaster::LDNS;
@@ -31,6 +32,8 @@ use List::Util qw[max];
 use Text::Reflow qw[reflow_string];
 use JSON::XS;
 use File::Slurp;
+use Net::Interface;
+use Socket qw[AF_INET AF_INET6];
 
 our %numeric = Zonemaster::Engine::Logger::Entry->levels;
 our $JSON    = JSON::XS->new->allow_blessed->convert_blessed->canonical;
@@ -144,7 +147,6 @@ has 'restore' => (
 has 'ipv4' => (
     is      => 'ro',
     isa     => 'Bool',
-    default => 1,
     documentation =>
       __( 'Flag to permit or deny queries being sent via IPv4. --ipv4 permits IPv4 traffic, --no-ipv4 forbids it.' ),
 );
@@ -152,7 +154,6 @@ has 'ipv4' => (
 has 'ipv6' => (
     is      => 'ro',
     isa     => 'Bool',
-    default => 1,
     documentation =>
       __( 'Flag to permit or deny queries being sent via IPv6. --ipv6 permits IPv6 traffic, --no-ipv6 forbids it.' ),
 );
@@ -317,11 +318,13 @@ sub run {
         print_test_list();
     }
 
-    Zonemaster::Engine::Profile->effective->set( q{net.ipv4}, 0+$self->ipv4 );
-    Zonemaster::Engine::Profile->effective->set( q{net.ipv6}, 0+$self->ipv6 );
-
     if ($self->sourceaddr) {
-        Zonemaster::Engine::Profile->effective->set( q{resolver.source}, $self->sourceaddr );
+        if ($self->check_sourceaddress_exists ) {
+            Zonemaster::Engine::Profile->effective->set( q{resolver.source}, $self->sourceaddr );
+        }
+        else {
+            die __x( "Address {address} cannot be used as source address for DNS queries.\n", address => $self->sourceaddr );
+        }
     }
 
     # Filehandle for diagnostics output
@@ -352,6 +355,16 @@ sub run {
         }
     }
 
+    # These two must come after any profile from command line has been loaded
+    # to make any IPv4/IPv6 option override the profile setting.
+    if ( defined ($self->ipv4) ) {
+        Zonemaster::Engine::Profile->effective->set( q{net.ipv4}, 0+$self->ipv4 );
+    }
+    if ( defined ($self->ipv6) ) {
+        Zonemaster::Engine::Profile->effective->set( q{net.ipv6}, 0+$self->ipv6 );
+    }
+
+
     if ( $self->dump_profile ) {
         do_dump_profile();
     }
@@ -434,16 +447,23 @@ sub run {
                     # Don't do anything
                 }
                 else {
-                    my $str = sprintf "%7.2f %-9s ", $entry->timestamp, $entry->level;
+                    my $prefix = sprintf "%7.2f %-9s ", $entry->timestamp, $entry->level;
                     if ( $self->show_module ) {
-                        $str.= sprintf "%-12s ", $entry->module;
+                        $prefix .= sprintf "%-12s ", $entry->module;
                     }
                     if ( $self->show_testcase ) {
-                        $str.= sprintf "%-14s ", $entry->testcase;
+                        $prefix .= sprintf "%-14s ", $entry->testcase;
+                    }
+                    $prefix .= $entry->tag;
+
+                    my $message = $entry->string;
+                    $message =~ s/^[A-Z0-9:_]+//;    # strip MODULE:TAG, they're coming in $prefix instead
+                    my @lines = split /\n/, $message;
+
+                    printf "%s%s %s\n", $prefix, ' ', shift @lines;
+                    for my $line ( @lines ) {
+                        printf "%s%s %s\n", $prefix, '>', $line;
                     }
-                    my $entry_str = sprintf "%s", $entry->string;
-                    $entry_str =~ s/^([A-Z0-9]+:)*//;
-                    printf "%s%s\n", $str, $entry_str;
                 }
             } ## end if ( $numeric{ uc $entry...})
             if ( $self->stop_level and $numeric{ uc $entry->level } >= $numeric{ $self->stop_level } ) {
@@ -578,6 +598,32 @@ sub run {
     return;
 } ## end sub run
 
+sub check_sourceaddress_exists {
+    my ( $self ) = @_;
+    my $address = Zonemaster::Engine::Net::IP->new($self->sourceaddr);
+    my $exists = 0;
+    foreach my $if ( Net::Interface->interfaces() ) {
+        foreach my $family ( AF_INET, AF_INET6 ) {
+            foreach my $ifaddr ( $if->address($family) ) {
+                my $zm_ifaddr;
+                if ( $family == AF_INET ) {
+                    $zm_ifaddr = Zonemaster::Engine::Net::IP->new(Net::Interface::inet_ntoa($ifaddr));
+                }
+                elsif ( $family == AF_INET6 ) {
+                    $zm_ifaddr = Zonemaster::Engine::Net::IP->new(Net::Interface::inet_ntop($ifaddr));
+                }
+                if ( $address->short eq $zm_ifaddr->short ) {
+                    $exists = 1;
+                    last;
+                }
+            }
+            last if $exists;
+        }
+        last if $exists;
+    }
+    return $exists;
+}
+
 sub add_fake_delegation {
     my ( $self, $domain ) = @_;
     my @ns_with_no_ip;
